@@ -7,6 +7,4 @@ POS = 'pos'
 LEMMA = 'lemma'
 DEPPARSE = 'depparse'
 NER = 'ner'
-
-# supported external packages
-SUPPORTED_TOKENIZERS = ['spacy', 'jieba']
+SENTIMENT = 'sentiment'
